@@ -57,12 +57,10 @@ try:
 except ImportError:
     import simplejson as json
 
-_VALID_VERSIONS = ['v1', 'v2', 'v3']
+_VALID_VERSIONS = ['v2', 'v3']
 V3_SERVICE_TYPE = 'volumev3'
 V2_SERVICE_TYPE = 'volumev2'
-V1_SERVICE_TYPE = 'volume'
-SERVICE_TYPES = {'1': V1_SERVICE_TYPE,
-                 '2': V2_SERVICE_TYPE,
+SERVICE_TYPES = {'2': V2_SERVICE_TYPE,
                  '3': V3_SERVICE_TYPE}
 REQ_ID_HEADER = 'X-OpenStack-Request-ID'
 
@@ -72,10 +70,14 @@ for svc in ('volume', 'volumev2', 'volumev3'):
     discover.add_catalog_discover_hack(svc, re.compile(r'/v[12]/\w+/?$'), '/')
 
 
-def get_server_version(url):
+def get_server_version(url, insecure=False, cacert=None):
     """Queries the server via the naked endpoint and gets version info.
 
     :param url: url of the cinder endpoint
+    :param insecure: Explicitly allow client to perform "insecure" TLS
+                     (https) requests
+    :param cacert: Specify a CA bundle file to use in verifying a TLS
+                            (https) server certificate
     :returns: APIVersion object for min and max version supported by
               the server
     """
@@ -89,11 +91,11 @@ def get_server_version(url):
 
         # NOTE(andreykurilin): endpoint URL has at least 2 formats:
         #   1. The classic (legacy) endpoint:
-        #       http://{host}:{optional_port}/v{1 or 2 or 3}/{project-id}
-        #       http://{host}:{optional_port}/v{1 or 2 or 3}
+        #       http://{host}:{optional_port}/v{2 or 3}/{project-id}
+        #       http://{host}:{optional_port}/v{2 or 3}
         #   3. Under wsgi:
-        #       http://{host}:{optional_port}/volume/v{1 or 2 or 3}
-        for ver in ['v1', 'v2', 'v3']:
+        #       http://{host}:{optional_port}/volume/v{2 or 3}
+        for ver in ['v2', 'v3']:
             if u.path.endswith(ver) or "/{0}/".format(ver) in u.path:
                 path = u.path[:u.path.rfind(ver)]
                 version_url = '%s://%s%s' % (u.scheme, u.netloc, path)
@@ -106,7 +108,14 @@ def get_server_version(url):
             # leave as is without cropping.
             version_url = url
 
-        response = requests.get(version_url)
+        if insecure:
+            verify_cert = False
+        else:
+            if cacert:
+                verify_cert = cacert
+            else:
+                verify_cert = True
+        response = requests.get(version_url, verify=verify_cert)
         data = json.loads(response.text)
         versions = data['versions']
         for version in versions:
@@ -114,6 +123,11 @@ def get_server_version(url):
                 min_version = version['min_version']
                 current_version = version['version']
                 break
+            else:
+                # Set the values, but don't break out the loop here in case v3
+                # comes later
+                min_version = '2.0'
+                current_version = '2.0'
     except exceptions.ClientException as e:
         logger.warning("Error in server version query:%s\n"
                        "Returning APIVersion 2.0", six.text_type(e.message))
@@ -121,9 +135,9 @@ def get_server_version(url):
             api_versions.APIVersion(current_version))
 
 
-def get_highest_client_server_version(url):
+def get_highest_client_server_version(url, insecure=False, cacert=None):
     """Returns highest supported version by client and server as a string."""
-    min_server, max_server = get_server_version(url)
+    min_server, max_server = get_server_version(url, insecure, cacert)
     max_client = api_versions.APIVersion(api_versions.MAX_VERSION)
     return min(max_server, max_client).get_string()
 
@@ -740,7 +754,6 @@ def _get_client_class_and_version(version):
 
 def get_client_class(version):
     version_map = {
-        '1': 'cinderclient.v1.client.Client',
         '2': 'cinderclient.v2.client.Client',
         '3': 'cinderclient.v3.client.Client',
     }
@@ -808,8 +821,7 @@ def Client(version, *args, **kwargs):
 
     Here ``VERSION`` can be a string or
     ``cinderclient.api_versions.APIVersion`` obj. If you prefer string value,
-    you can use ``1`` (deprecated now), ``2``, or ``3.X``
-    (where X is a microversion).
+    you can use ``2`` (deprecated now) or ``3.X`` (where X is a microversion).
 
 
     Alternatively, you can create a client instance using the keystoneclient
