@@ -46,11 +46,13 @@ import six
 from six.moves.urllib import parse
 
 import cinderclient
+from cinderclient import api_versions
 from cinderclient import base
 from cinderclient import client
 from cinderclient import exceptions
 from cinderclient import shell
 from cinderclient import utils as cinderclient_utils
+from cinderclient.v3 import attachments
 from cinderclient.v3 import volume_snapshots
 from cinderclient.v3 import volumes
 
@@ -79,6 +81,8 @@ class ShellTest(utils.TestCase):
             self.useFixture(fixtures.EnvironmentVariable(var,
                                                          self.FAKE_ENV[var]))
 
+        self.mock_completion()
+
         self.shell = shell.OpenStackCinderShell()
 
         self.requests = self.useFixture(requests_mock_fixture.Fixture())
@@ -89,7 +93,12 @@ class ShellTest(utils.TestCase):
         self.cs = mock.Mock()
 
     def run_command(self, cmd):
-        self.shell.main(cmd.split())
+        # Ensure the version negotiation indicates that
+        # all versions are supported
+        with mock.patch('cinderclient.api_versions._get_server_version_range',
+                return_value=(api_versions.APIVersion('3.0'),
+                              api_versions.APIVersion('3.99'))):
+            self.shell.main(cmd.split())
 
     def assert_called(self, method, url, body=None,
                       partial_body=None, **kwargs):
@@ -136,32 +145,45 @@ class ShellTest(utils.TestCase):
              u'list --filters name~=Σ',
          'expected':
              '/volumes/detail?name~=%CE%A3'},
+        {'command':
+             u'list --filters name=abc --filters size=1',
+         'expected':
+             '/volumes/detail?name=abc&size=1'},
         # testcases for list group
         {'command':
              'group-list --filters name=456',
          'expected':
-             '/groups/detail?all_tenants=0&name=456'},
+             '/groups/detail?name=456'},
         {'command':
              'group-list --filters status=available',
          'expected':
-             '/groups/detail?all_tenants=0&status=available'},
+             '/groups/detail?status=available'},
         {'command':
              'group-list --filters name~=456',
          'expected':
-             '/groups/detail?all_tenants=0&name~=456'},
+             '/groups/detail?name~=456'},
+        {'command':
+             'group-list --filters name=abc --filters status=available',
+         'expected':
+             '/groups/detail?name=abc&status=available'},
         # testcases for list group-snapshot
         {'command':
              'group-snapshot-list --status=error --filters status=available',
          'expected':
-             '/group_snapshots/detail?all_tenants=0&status=available'},
+             '/group_snapshots/detail?status=available'},
         {'command':
              'group-snapshot-list --filters availability_zone=123',
          'expected':
-             '/group_snapshots/detail?all_tenants=0&availability_zone=123'},
+             '/group_snapshots/detail?availability_zone=123'},
         {'command':
              'group-snapshot-list --filters status~=available',
          'expected':
-             '/group_snapshots/detail?all_tenants=0&status~=available'},
+             '/group_snapshots/detail?status~=available'},
+        {'command':
+             'group-snapshot-list --filters status=available '
+             '--filters availability_zone=123',
+         'expected':
+             '/group_snapshots/detail?availability_zone=123&status=available'},
         # testcases for list message
         {'command':
              'message-list --event_id=123 --filters event_id=456',
@@ -175,6 +197,10 @@ class ShellTest(utils.TestCase):
              'message-list --filters request_id~=123',
          'expected':
              '/messages?request_id~=123'},
+        {'command':
+             'message-list --filters request_id=123 --filters event_id=456',
+         'expected':
+             '/messages?event_id=456&request_id=123'},
         # testcases for list attachment
         {'command':
              'attachment-list --volume-id=123 --filters volume_id=456',
@@ -188,6 +214,11 @@ class ShellTest(utils.TestCase):
              'attachment-list --filters volume_id~=456',
          'expected':
              '/attachments?volume_id~=456'},
+        {'command':
+             'attachment-list --filters volume_id=123 '
+             '--filters mountpoint=456',
+         'expected':
+             '/attachments?mountpoint=456&volume_id=123'},
         # testcases for list backup
         {'command':
              'backup-list --volume-id=123 --filters volume_id=456',
@@ -201,6 +232,10 @@ class ShellTest(utils.TestCase):
              'backup-list --filters volume_id~=456',
          'expected':
              '/backups/detail?volume_id~=456'},
+        {'command':
+             'backup-list --filters volume_id=123 --filters name=456',
+         'expected':
+             '/backups/detail?name=456&volume_id=123'},
         # testcases for list snapshot
         {'command':
              'snapshot-list --volume-id=123 --filters volume_id=456',
@@ -214,6 +249,10 @@ class ShellTest(utils.TestCase):
              'snapshot-list --filters volume_id~=456',
          'expected':
              '/snapshots/detail?volume_id~=456'},
+        {'command':
+             'snapshot-list --filters volume_id=123 --filters name=456',
+         'expected':
+             '/snapshots/detail?name=456&volume_id=123'},
         # testcases for get pools
         {'command':
              'get-pools --filters name=456 --detail',
@@ -222,7 +261,11 @@ class ShellTest(utils.TestCase):
         {'command':
              'get-pools --filters name=456',
          'expected':
-             '/scheduler-stats/get_pools?name=456'}
+             '/scheduler-stats/get_pools?name=456'},
+        {'command':
+             'get-pools --filters name=456 --filters detail=True',
+         'expected':
+             '/scheduler-stats/get_pools?detail=True&name=456'}
     )
     @ddt.unpack
     def test_list_with_filters_mixed(self, command, expected):
@@ -263,7 +306,17 @@ class ShellTest(utils.TestCase):
                     {six.text_type('key'): six.text_type('value')}}))
         self.assert_call_contained(parse.urlencode({'is_public': None}))
 
-    def test_type_list_no_filters(self):
+    def test_type_list_public(self):
+        self.run_command('--os-volume-api-version 3.52 type-list '
+                         '--filters is_public=True')
+        self.assert_called('GET', '/types?is_public=True')
+
+    def test_type_list_private(self):
+        self.run_command('--os-volume-api-version 3.52 type-list '
+                         '--filters is_public=False')
+        self.assert_called('GET', '/types?is_public=False')
+
+    def test_type_list_public_private(self):
         self.run_command('--os-volume-api-version 3.52 type-list')
         self.assert_called('GET', '/types?is_public=None')
 
@@ -282,6 +335,14 @@ class ShellTest(utils.TestCase):
         mock_print.assert_called_once_with(mock.ANY, key_list,
             exclude_unavailable=True, sortby_index=0)
 
+    @mock.patch("cinderclient.shell.OpenStackCinderShell.downgrade_warning")
+    def test_list_version_downgrade(self, mock_warning):
+        self.run_command('--os-volume-api-version 3.998 list')
+        mock_warning.assert_called_once_with(
+            api_versions.APIVersion('3.998'),
+            api_versions.APIVersion(api_versions.MAX_VERSION)
+        )
+
     def test_list_availability_zone(self):
         self.run_command('availability-zone-list')
         self.assert_called('GET', '/os-availability-zone')
@@ -402,6 +463,21 @@ class ShellTest(utils.TestCase):
         self.run_command(command)
         self.assert_called('GET', '/attachments%s' % expected)
 
+    @mock.patch('cinderclient.utils.print_list')
+    @mock.patch.object(cinderclient.v3.attachments.VolumeAttachmentManager,
+            'list')
+    def test_attachment_list_setattr(self, mock_list, mock_print):
+        command = '--os-volume-api-version 3.27 attachment-list '
+        fake_attachment = [attachments.VolumeAttachment(mock.ANY, attachment)
+                for attachment in fakes.fake_attachment_list['attachments']]
+        mock_list.return_value = fake_attachment
+        self.run_command(command)
+        for attach in fake_attachment:
+            setattr(attach, 'server_id', getattr(attach, 'instance'))
+        columns = ['ID', 'Volume ID', 'Status', 'Server ID']
+        mock_print.assert_called_once_with(fake_attachment, columns,
+                sortby_index=0)
+
     def test_revert_to_snapshot(self):
         original = cinderclient_utils.find_resource
 
@@ -553,6 +629,20 @@ class ShellTest(utils.TestCase):
         self.run_command('--os-volume-api-version 3.11 group-type-list')
         self.assert_called_anytime('GET', '/group_types?is_public=None')
 
+    def test_group_type_list_public(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list '
+                         '--filters is_public=True')
+        self.assert_called('GET', '/group_types?is_public=True')
+
+    def test_group_type_list_private(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list '
+                         '--filters is_public=False')
+        self.assert_called('GET', '/group_types?is_public=False')
+
+    def test_group_type_list_public_private(self):
+        self.run_command('--os-volume-api-version 3.52 group-type-list')
+        self.assert_called('GET', '/group_types?is_public=None')
+
     def test_group_type_show(self):
         self.run_command('--os-volume-api-version 3.11 '
                          'group-type-show 1')
@@ -602,7 +692,6 @@ class ShellTest(utils.TestCase):
                                'metadata': {},
                                'volume_type': '4321',
                                'description': None,
-                               'multiattach': False,
                                'backup_id': None}}
         self.assert_called_anytime('POST', '/volumes', expected)
 
@@ -625,14 +714,13 @@ class ShellTest(utils.TestCase):
                                'metadata': {},
                                'volume_type': None,
                                'description': None,
-                               'multiattach': False,
                                'backup_id': None}}
         expected['volume'].update(update)
         self.assert_called_anytime('POST', '/volumes', body=expected)
 
     def test_group_list(self):
         self.run_command('--os-volume-api-version 3.13 group-list')
-        self.assert_called_anytime('GET', '/groups/detail?all_tenants=0')
+        self.assert_called_anytime('GET', '/groups/detail')
 
     def test_group_list__with_all_tenant(self):
         self.run_command(
@@ -692,7 +780,7 @@ class ShellTest(utils.TestCase):
     def test_group_snapshot_list(self):
         self.run_command('--os-volume-api-version 3.14 group-snapshot-list')
         self.assert_called_anytime('GET',
-                                   '/group_snapshots/detail?all_tenants=0')
+                                   '/group_snapshots/detail')
 
     def test_group_snapshot_show(self):
         self.run_command('--os-volume-api-version 3.14 '
@@ -1349,3 +1437,9 @@ class ShellTest(utils.TestCase):
                                  'no_snapshots': True
                                  }}
         self.assert_called('POST', '/volume-transfers', body=expected)
+
+    def test_list_transfer_sorty_not_sorty(self):
+        self.run_command(
+            '--os-volume-api-version 3.59 transfer-list')
+        url = ('/volume-transfers/detail')
+        self.assert_called('GET', url)
